package me.paradoxpixel.themepark.attraction;

import me.paradoxpixel.themepark.ThemeParkPlugin;
import me.paradoxpixel.themepark.action.PageAction;
import me.paradoxpixel.themepark.action.TPUtils;
import me.paradoxpixel.themepark.api.API;
import me.paradoxpixel.themepark.api.attraction.Attraction;
import me.paradoxpixel.themepark.api.attraction.Region;
import me.paradoxpixel.themepark.api.attraction.component.Status;
import me.paradoxpixel.themepark.api.attraction.component.Type;
import me.paradoxpixel.themepark.attraction.status.StatusManager;
import me.paradoxpixel.themepark.config.YamlConfig;
import me.paradoxpixel.themepark.gui.GUI;
import me.paradoxpixel.themepark.gui.GUIItem;
import me.paradoxpixel.themepark.utils.ItemBuilder;
import me.paradoxpixel.themepark.api.LocationUtils;
import me.paradoxpixel.themepark.utils.Utils;
import org.bukkit.Location;
import org.bukkit.Material;
import org.bukkit.configuration.ConfigurationSection;
import org.bukkit.entity.Player;
import java.util.HashMap;
import java.util.List;
import java.util.Set;

public class AttractionMenu {

    private static YamlConfig config = ThemeParkPlugin.getInstance().getAttraction();
    private static YamlConfig settings = ThemeParkPlugin.getInstance().getSettings();
    private static HashMap<Integer, GUI> pages = new HashMap<>();
    private static boolean loading = false;
    private static String title;

    private static HashMap<String, Integer> regionPage;
    private static HashMap<String, Integer> attractionIndex;

    private static Material mat = Material.NAME_TAG;
    private static short data = 0;

    public static void load() {
        loading = true;
        title = settings.getConfig().getString("menu.title");

        mat = Material.getMaterial(settings.getConfig().getString("region.material"));
        data = Short.parseShort(settings.getConfig().getString("region.data"));

        regionPage = new HashMap<>();
        attractionIndex = new HashMap<>();

        loadData();
        loading = false;
        loadItems();
    }

    public static void reload() {
        loadItems();
    }

    public static void save() {
        for(Region region : API.getRegions().values()) {
            config.getConfig().set("region." + region.getId() + ".name", region.getName());
            config.getConfig().set("region." + region.getId() + ".lore", region.getLore());
        }

        for(Attraction attraction : API.getAttractions().values()) {
            config.getConfig().set("attraction." + attraction.getId() + ".name", attraction.getName());
            config.getConfig().set("attraction." + attraction.getId() + ".region_id", attraction.getRegion_id());
            config.getConfig().set("attraction." + attraction.getId() + ".type", attraction.getType().toString());
            config.getConfig().set("attraction." + attraction.getId() + ".status", attraction.getStatus().toString());
            config.getConfig().set("attraction." + attraction.getId() + ".location", LocationUtils.toString(attraction.getLocation()));
        }

        config.save();
    }

    private static void loadData() {
        if(!config.getConfig().contains("region"))
            return;

        if(!config.getConfig().isConfigurationSection("region"))
            return;

        ConfigurationSection section = config.getConfig().getConfigurationSection("region");
        if(section.getKeys(false).isEmpty())
            return;

        HashMap<String, Region> regions = API.getRegions();
        HashMap<String, Integer> size = new HashMap<>();
        for(String id : section.getKeys(false)) {
            String name = config.getConfig().getString("region." + id + ".name");
            List<String> lore = config.getConfig().getStringList("region." + id + ".lore");

            size.put(id.toLowerCase(), 0);
            regions.remove(id);
            if(API.isRegion(id)) {
                API.getRegion(id).setName(name);
                API.getRegion(id).setLore(lore);
                continue;
            }

            Region region = new Region(id, name, lore);
            API.addRegion(region);
            if(!id.toLowerCase().equals(id)) {
                config.getConfig().set("region." + id, null);
                id = id.toLowerCase();
                config.getConfig().set("region." + id + ".name", name);
                config.getConfig().set("region." + id + ".lore", lore);
            }
        }

        config.save();
        for(String string : regions.keySet())
            API.removeRegion(string);

        prepareGUI();
        if(!config.getConfig().contains("attraction"))
            return;

        if(!config.getConfig().isConfigurationSection("attraction"))
            return;

        section = config.getConfig().getConfigurationSection("attraction");
        if(section.getKeys(false).isEmpty())
            return;

        HashMap<String, Attraction> attractions = API.getAttractions();
        for(String id : section.getKeys(false)) {
            String region_id = config.getConfig().getString("attraction." + id + ".region_id");
            if(region_id == null || region_id.isEmpty())
                continue;

            if(!region_id.toLowerCase().equals(region_id)) {
                region_id = region_id.toLowerCase();
                config.getConfig().set("attraction." + id + ".region_id", region_id);
            }

            if(!API.isRegion(region_id))
                continue;

            if(size.get(region_id.toLowerCase()) >= 8)
                continue;

            String name = config.getConfig().getString("attraction." + id + ".name");
            Location location = LocationUtils.toLocation(config.getConfig().getString("attraction." + id + ".location"));
            Type type = Type.getType(config.getConfig().getString("attraction." + id + ".type"));
            Status status = Status.getStatus(config.getConfig().getString("attraction." + id + ".status"));
            if(!type.containsStatus(status)) {
                status = type.getDefault();
                config.getConfig().set("attraction." + id + ".status", type.getDefault().toString());
            }

            attractions.remove(id.toLowerCase());
            size.put(region_id.toLowerCase(), size.get(region_id.toLowerCase()) + 1);
            if(API.isAttraction(id)) {
                Attraction attraction = API.getAttraction(id);
                attraction.setName(name);
                attraction.setRegion_id(region_id);
                attraction.setLocation(location);
                attraction.setType(type);
                attraction.setStatus(status, null);
                MapMarker.getMarker().setAttractionMarker(attraction);
                continue;
            }

            Attraction attraction = new Attraction(id, name, region_id, location, type, status);
            API.addAttraction(attraction);
            MapMarker.getMarker().setAttractionMarker(attraction);
            if(!id.toLowerCase().equals(id)) {
                config.getConfig().set("attraction." + id, null);
                id = id.toLowerCase();
                config.getConfig().set("attraction." + id + ".name", name);
                config.getConfig().set("attraction." + id + ".location", LocationUtils.toString(location));
                config.getConfig().set("attraction." + id + ".type", type.toString());
                config.getConfig().set("attraction." + id + ".status", status.toString());
            }
        }

        config.save();
        for(String string : attractions.keySet())
            API.removeAttraction(string);
    }

    private static void prepareGUI() {
        int keySize = API.getRegions().size();
        if(pages.isEmpty()) {
            if(keySize <= 6) {
                pages.put(1, new GUI(title, keySize * 9));
            } else {
                int p = keySize / 5;
                for(int i = 0; i < p; i++)
                    pages.put(i + 1, new GUI(title, 54));

                int rest = keySize % 5;
                if(rest > 0)
                    pages.put(p + 1, new GUI(title, rest * 9 + 9));
            }
        } else {
            if(keySize <= 6) {
                if(pages.size() > 1) {
                    for(int i = 1; i < pages.size(); i++)
                        pages.remove(i + 1);
                }

                int z = keySize*9;
                GUI gui = pages.get(1);
                if(gui.getSize() != z)
                    gui.setSize(z);
            } else {
                int p = keySize / 5;
                int rest = keySize % 5;
                if (rest > 0)
                    p++;

                if (pages.size() < p) {
                    for (int i = (pages.size() + 1); i < p; i++)
                        pages.put(i, new GUI(title, 54));

                    pages.put(p, new GUI(title, rest * 9 + 9));
                } else {
                    if (pages.size() > p) {
                        for (int i = pages.size(); i > p; i--)
                            pages.remove(i);
                    } else {
                        if(rest != 0) {
                            pages.get(p).setSize(rest * 9 + 9);
                        } else {
                            pages.get(p).setSize(54);
                        }
                    }
                }
            }
        }
    }

    public static void update(Attraction attraction) {
        if(attraction == null)
            return;

        if(!API.isAttraction(attraction.getId()))
            return;

        if(!attractionIndex.containsKey(attraction.getId()))
            return;

        if(!regionPage.containsKey(attraction.getRegion_id()))
            return;

        ItemBuilder builder = new ItemBuilder(StatusManager.getMaterial(attraction.getStatus()), 1, StatusManager.getData(attraction.getStatus()));
        builder.setName(attraction.getName());
        builder.setLore(StatusManager.getName(attraction.getStatus()));

        GUI gui = pages.get(regionPage.get(attraction.getRegion_id()));
        gui.addItem(attractionIndex.get(attraction.getId()), new GUIItem(builder.getItem(), new TPUtils(attraction.getId()), true));
    }

    private static void loadItems() {
        if(loading)
            return;

        if(API.getRegions().values().size() == 0)
            return;

        for(GUI gui : pages.values())
            gui.clear();

        int i = 1;
        int j = 0;
        boolean tb = pages.size() > 1;
        HashMap<String, Integer> index = new HashMap<>();
        for(Region region : API.getRegions().values()) {
            if(tb && j >= 5) {
                i++;
                j = 0;
            }

            index.put(region.getId(), j * 9);
            regionPage.put(region.getId(), i);
            if(!index.containsKey(region.getId()))
                index.put(region.getId(), index.size() * 9);

            ItemBuilder builder = new ItemBuilder(mat, 1, data);
            builder.setName(region.getName());
            builder.setLore(region.getLore());

            GUI gui = pages.get(i);
            gui.addItem(j * 9, new GUIItem(builder.getItem(), null, true));
            j++;
        }


        for(Attraction attraction : API.getAttractions().values()) {
            if(!API.isRegion(attraction.getRegion_id())) {
                API.removeAttraction(attraction.getId());
                continue;
            }

            if(!index.containsKey(attraction.getRegion_id()))
                continue;

            int in = index.get(attraction.getRegion_id());
            GUI gui = pages.get(regionPage.get(attraction.getRegion_id()));
            boolean b = false;
            for(int a = 1; a < 9; a++) {
                if(!gui.hasItem(in + a)) {
                    in += a;
                    b = true;
                    break;
                }
            }

            if(!b) {
                API.removeAttraction(attraction.getId());
                continue;
            }

            ItemBuilder builder = new ItemBuilder(StatusManager.getMaterial(attraction.getStatus()), 1, StatusManager.getData(attraction.getStatus()));
            builder.setName(attraction.getName());
            builder.setLore(StatusManager.getName(attraction.getStatus()));

            gui.addItem(in, new GUIItem(builder.getItem(), new TPUtils(attraction.getId()), true));
            attractionIndex.put(attraction.getId(), in);
        }

        if(pages.size() <= 1)
            return;

        i = 1;
        for(GUI gui : pages.values()) {
            int rows = gui.getSize() / 9;
            int offset = (rows - 1) * 9;

            if(i > 1) {
                ItemBuilder previous = new ItemBuilder(Material.ARROW);
                previous.setName(Utils.color("&6Previous"));
                gui.addItem(offset + 3, new GUIItem(previous.getItem(), new PageAction(i - 1), true));
            }

            ItemBuilder middle = new ItemBuilder(Material.STAINED_GLASS_PANE, 1, (short) 1);
            middle.setName(Utils.color("&6Page: "+i));
            gui.addItem(offset + 4, new GUIItem(middle.getItem(), null, true));

            if(i < pages.size()) {
                ItemBuilder next = new ItemBuilder(Material.ARROW);
                next.setName(Utils.color("&6Next"));
                gui.addItem(offset + 5, new GUIItem(next.getItem(), new PageAction(i + 1), true));
            }

            i++;
        }
    }

    public static void openMenu(Player player, int page) {
        if(page < 1 || page > pages.size())
            page = 1;

        player.openInventory(pages.get(page).getInventory());
    }

}