package me.paradoxpixel.themepark.listener;

import me.paradoxpixel.themepark.ThemeParkPlugin;
import me.paradoxpixel.themepark.config.YamlConfig;
import me.paradoxpixel.themepark.database.DatabaseHandler;
import me.paradoxpixel.themepark.utils.ItemBuilder;
import me.paradoxpixel.themepark.utils.Utils;
import org.bukkit.Material;
import org.bukkit.entity.Minecart;
import org.bukkit.entity.Player;
import org.bukkit.event.EventHandler;
import org.bukkit.event.Listener;
import org.bukkit.event.player.PlayerChangedWorldEvent;
import org.bukkit.event.player.PlayerJoinEvent;
import org.bukkit.event.player.PlayerTeleportEvent;
import org.bukkit.inventory.ItemStack;

import java.util.List;

public class PlayerListener implements Listener {

    private YamlConfig config = ThemeParkPlugin.getInstance().getData();
    private YamlConfig settings = ThemeParkPlugin.getInstance().getSettings();

    @EventHandler
    public void onJoin(PlayerJoinEvent e) {
        doItem(e.getPlayer());
    }

    @EventHandler
    public void onSwitch(PlayerChangedWorldEvent e) {
        doItem(e.getPlayer());
    }

    @EventHandler
    public void onTeleport(PlayerTeleportEvent e) {
        Player player = e.getPlayer();
        if(e.getCause() != PlayerTeleportEvent.TeleportCause.PLUGIN)
            return;

        if(!player.isInsideVehicle())
            return;

        if(player.getVehicle() == null)
            return;

        if(!(player.getVehicle() instanceof Minecart))
            return;

        e.setCancelled(true);
        player.sendMessage(Utils.color("&6ThemePark&f: &4You can't teleport while in a minecart"));
    }

    public void doItem(Player player) {
        if(!settings.getConfig().getBoolean("item.enabled"))
            return;

        Material material = Material.getMaterial(settings.getConfig().getString("item.material"));
        String name = Utils.color(settings.getConfig().getString("item.display-name"));
        int slot = settings.getConfig().getInt("item.slot");
        if(material == null || name.isEmpty())
            return;

        boolean b = true;
        if(config.getConfig().contains(player.getUniqueId().toString() + ".item")) {
            b = config.getConfig().getBoolean(player.getUniqueId().toString() + ".item");
        } else {
            config.getConfig().set(player.getUniqueId().toString() + ".item", true);
            config.save();
        }

        if(b) {
            if(settings.getConfig().getBoolean("inventory.clear"))
                player.getInventory().clear();

            List<String> worlds = settings.getConfig().getStringList("item.worlds");
            if(worlds != null && !worlds.isEmpty()) {
                if (worlds.contains(player.getLocation().getWorld().getName())) {
                    player.getInventory().setItem(slot, new ItemStack(Material.AIR));
                    return;
                }
            }

            ItemBuilder builder = new ItemBuilder(material);
            builder.setName(name);
            player.getInventory().setItem(slot, builder.getItem());
        }
    }

}